@@ -1,4 +1,4 @@
-# .. Copyright (C) 2012-2019 Bryan A. Jones.
+# .. Copyright (C) 2012-2020 Bryan A. Jones.
 #
 #    This file is part of CodeChat.
 #
@@ -32,6 +32,7 @@
 #
 # Standard library
 # ----------------
+import html
 from io import StringIO
 import os.path
 from pathlib import Path
@@ -46,7 +47,7 @@ from docutils.writers.html4css1 import Writer
 from docutils.parsers.rst import states
 from docutils import statemachine, utils
 from docutils.utils.error_reporting import SafeString, ErrorString
-
+import pkg_resources
 
 # Local application imports
 # -------------------------
@@ -120,7 +121,7 @@ def code_to_rest_file(
     fi = io.FileInput(source_path=source_path, encoding=input_encoding)
     fo = io.FileOutput(destination_path=rst_path, encoding=output_encoding)
     code_str = fi.read()
-    # If not already present, provde the filename of the source to help in identifying a lexer.
+    # If not already present, provide the filename of the source to help in identifying a lexer.
     options.setdefault("filename", source_path)
     rst = code_to_rest_string(code_str, **options)
     fo.write(rst)
@@ -153,8 +154,7 @@ def code_to_html_string(
             # Include our custom css file: provide the path to the default css and
             # then to our css. The style sheet dirs must include docutils defaults.
             "stylesheet_path": ",".join(Writer.default_stylesheets + ["CodeChat.css"]),
-            "stylesheet_dirs": Writer.default_stylesheet_dirs
-            + [os.path.join(os.path.dirname(__file__), "template")],
+            "stylesheet_dirs": Writer.default_stylesheet_dirs + html_static_path(),
             # Make sure to use Unicode everywhere.
             "output_encoding": "unicode",
             "input_encoding": "unicode",
@@ -195,6 +195,14 @@ def code_to_html_file(
     fo.write(html)
 
 
+# Styling
+# --------
+# Provide a programmatic way to get a list of paths to static files needed by CodeChat. When using Sphinx, this should be assigned or appended to `html_static_path <http://sphinx-doc.org/config.html#confval-html_static_path>`_.
+def html_static_path():
+    # There's only one path needed -- ``css/``, relative to this directory.
+    return [pkg_resources.resource_filename("CodeChat", "css")]
+
+
 # Provide correct formatting of CodeChat-produced documents in reST based on the `CodeChat style`.
 rest_codechat_style = ".. raw:: html\n" "\n" " " + codechat_style
 
@@ -673,7 +681,7 @@ def _docname_role(
     return [nodes.inline(rawtext, path_component, **options)], []
 
 
-# .. add_highlight_language:
+# .. _add_highlight_language:
 #
 # add_highlight_language
 # ^^^^^^^^^^^^^^^^^^^^^^
@@ -701,9 +709,9 @@ def add_highlight_language(
 #
 # _CodeInclude
 # ^^^^^^^^^^^^
-# Provide a way to include source code to be processed by CodeChat.
+# Provide a way to include source code to be processed by CodeChat. It is a slight modification of the `docutils include directive <https://docutils.sourceforge.io/docs/ref/rst/directives.html#including-an-external-document-fragment>`_ and supports all the same options; it also supports the `class <https://docutils.sourceforge.io/docs/ref/rst/directives.html#id22>`_ option.
 #
-# This is mostly copied directly from ``docutils.parsers.rst.directives.misc.Include``.
+# Implementation note: this is mostly copied directly from ``docutils.parsers.rst.directives.misc.Include``, version 0.16.
 class _CodeInclude(Directive):
 
     """
@@ -728,6 +736,7 @@ class _CodeInclude(Directive):
         "end-line": int,
         "start-after": directives.unchanged_required,
         "end-before": directives.unchanged_required,
+        "class": directives.class_option,
     }
 
     standard_include_path = os.path.join(os.path.dirname(states.__file__), "include")
@@ -760,13 +769,13 @@ class _CodeInclude(Directive):
             )
         except UnicodeEncodeError as error:
             raise self.severe(
-                'Problems with "%s" directive path:\n'
+                u'Problems with "%s" directive path:\n'
                 'Cannot encode input file path "%s" '
                 "(wrong locale?)." % (self.name, SafeString(path))
             )
         except IOError as error:
             raise self.severe(
-                'Problems with "%s" directive path:\n%s.'
+                u'Problems with "%s" directive path:\n%s.'
                 % (self.name, ErrorString(error))
             )
         startline = self.options.get("start-line", None)
@@ -779,7 +788,7 @@ class _CodeInclude(Directive):
                 rawtext = include_file.read()
         except UnicodeError as error:
             raise self.severe(
-                'Problem with "%s" directive:\n%s' % (self.name, ErrorString(error))
+                u'Problem with "%s" directive:\n%s' % (self.name, ErrorString(error))
             )
         # start-after/end-before: no restrictions on newlines in match-text,
         # and no restrictions on matching inside lines vs. line boundaries
@@ -825,6 +834,19 @@ class _CodeInclude(Directive):
         lexer = get_lexer(filename=path, code=rawtext, **code_to_rest_options)
         rawtext = code_to_rest_string(rawtext, lexer=lexer)
 
+        # If the ``class`` option is specified, wrap the code in a div with the specified classes.
+        classes = self.options.get("class")
+        if classes:
+            rawtext = (
+                ".. raw:: html\n"
+                "\n"
+                " <div class='" + html.escape(" ".join(classes)) + "'>\n"
+                "\n" + rawtext + "\n"
+                ".. raw:: html\n"
+                "\n"
+                " </div>\n"
+            )
+
         # If Sphinx is running, insert the appropriate highlight directive.
         if env:
             rawtext = add_highlight_language(rawtext, lexer)
