@@ -1,4 +1,4 @@
-# .. Copyright (C) 2012-2019 Bryan A. Jones.
+# .. Copyright (C) 2012-2020 Bryan A. Jones.
 #
 #    This file is part of CodeChat.
 #
@@ -18,7 +18,7 @@
 # ***************************************
 # |docname| - a module to lex source code
 # ***************************************
-# This module classifies source code in any format supported by `CommentDelimiterInfo.py` into an interable of ``(type, string)`` tuples; see `source_lexer`.
+# This module classifies source code in any format supported by `CommentDelimiterInfo.py` into an iterable of ``(type, string)`` tuples; see `source_lexer`.
 #
 # .. contents::
 #
@@ -41,7 +41,7 @@ from pygments.lexers import (
     guess_lexer_for_filename,
     guess_lexer,
 )
-from pygments.util import text_type, guess_decode
+from pygments.util import guess_decode
 from pygments.lexer import _encoding_map
 from pygments.token import Token
 from pygments.filter import apply_filters
@@ -158,25 +158,48 @@ def source_lexer(
 #
 # CodeChat style
 # --------------
-# `../CodeChat.css` provides some of the CSS needed to properly format CodeChat documents. However, not all the necessary styling can be accomplished using CSS. This script sets the styles that can't be set in CSS. Specifically, it removes the bottom margin for cases where code follows a paragraph. The expected structure:
+# `css/CodeChat.css` provides some of the CSS needed to properly format CodeChat documents. However, not all the necessary styling can be accomplished using CSS. This script sets the styles that can't be set in CSS. Specifically, it removes the bottom margin for cases where code follows a paragraph, and the top margin for cases where code preceds a paragraph. The expected structure:
 #
 # .. code-block:: HTML
 #   :linenos:
 #
-#   <div class="CodeChat-indent">    An indent from CodeChat -- not present with no indent.
-#       <p>Text.</p>                 DO NOT change this element's bottom margin.
-#       <p>Some text here</p>        This could be any element.
-#   </div>
-#   <div class="highlight-xxx">      Where xxx is the language name, such as c, python, etc.
+#   <!-- All CodeChat-produced code is marked by the ``fenced-code``
+#        class. -->
+#   <div class="fenced-code">
 #       ...Some code here...
 #   </div>
+#
+#   <!-- An indent from CodeChat -- not present with no indent. The
+#        code below will add the classes ``CodeChat_noTop
+#        CodeChat_noBottom`` to this element.
+#   -->
+#   <div class="CodeChat-indent">
+#       <!-- The code below will add the class ``CodeChat_noTop`` to
+#            this element.
+#       -->
+#       <p>Text.</p>
+#       <!-- The code below will add the class ``CodeChat_noBottom``
+#            to this element.
+#       -->
+#       <p>Some text here</p>
+#   </div>
+#
+#   <div class="fenced-code">
+#       ...Some more code here...
+#   </div>
+#
+# `Disable Black <https://github.com/psf/black#the-black-code-style>`_ for this block.
+# fmt: off
 codechat_style = (
     '<script type="text/javascript">'
-    # Only style after the `DOM is ready <https://stackoverflow.com/questions/799981/document-ready-equivalent-without-jquery>`_.
-    'document.addEventListener("DOMContentLoaded", function(event) {'
+    # Define a function to add classes as defined above.
+    "function CodeChat_doStyle("
+        # It takes an optional parameter, the root element to search from. Styles will be applies to this element and all its children. It defaults to ``document``.
+        "element"
+    ") {"
         # Walk the tree in the given direction.
-        "let walk_tree = function("
-            # The jQuery elements to walk.
+        "function walk_tree("
+            # A list of DOM elements to walk.
             "elements,"
             # The walker function: ``x => x.next/prevElementSibling``.
             "func_walk,"
@@ -208,21 +231,25 @@ codechat_style = (
             "return walk_children;"
         "};"
 
+        # Use ``document`` as the default element if it isn't specified.
+        "element = (typeof element !== 'undefined') ? element : document;"
         # All CodeChat-produced code is marked by the ``fenced-code`` class.
-        'let code = document.getElementsByClassName("fenced-code");'
+        'let code = element.getElementsByClassName("fenced-code");'
         # Go to the next node from code, then set the margin-top of all first children to 0 so that there will be no extra space between the code and the following comment.
-        "walk_tree(code, x => x.nextElementSibling, x => x.firstElementChild).map(x => {"
-            "x.style.marginTop = 0;"
-            "x.style.paddingTop = 0;"
-        "});"
+        "walk_tree(code, x => x.nextElementSibling, x => x.firstElementChild).map(x =>"
+            "x.classList.add('CodeChat_noTop')"
+        ");"
         # Same, but remove space between a comment and the following code.
-        "walk_tree(code, x => x.previousElementSibling, x => x.lastElementChild).map(x => {"
-            "x.style.marginBottom = 0;"
-            "x.style.paddingBottom = 0;"
-        "});"
-    "});"
+        "walk_tree(code, x => x.previousElementSibling, x => x.lastElementChild).map(x =>"
+            "x.classList.add('CodeChat_noBottom')"
+        ");"
+    "}"
+
+    # Only style after the `DOM is ready <https://stackoverflow.com/questions/799981/document-ready-equivalent-without-jquery>`_.
+    'document.addEventListener("DOMContentLoaded", function(event) { CodeChat_doStyle(); });'
     "</script>"
 )
+# fmt: on
 
 
 # Step 1 of source_lexer_
@@ -301,7 +328,7 @@ def _pygments_lexer(
     # So, process this with AST if this is Python or Python3 code to find docstrings.
     # If found, store line number and docstring into ``ast_docstring``.
     ast_docstring = {}
-    # Provide a place to store syntax errors resutling from parsing the Python code.
+    # Provide a place to store syntax errors resulting from parsing the Python code.
     ast_syntax_error = ""
     # Determine if code is Python or Python3. Note that AST processing cannot
     # support Python 2 specific syntax (e.g. the ``<>`` operator).
@@ -339,7 +366,7 @@ def _pygments_lexer(
 # Pygments monkeypatching
 # ^^^^^^^^^^^^^^^^^^^^^^^
 # Provide a way to perform preprocessing on text before lexing it. This code was
-# copied from ``pygments.lexer.Lexer.get_token``, v. 2.1.3.
+# copied from ``pygments.lexer.Lexer.get_token``, v. 2.6.1.
 def _pygments_get_tokens_preprocess(self, text, unfiltered=False):
     """
     Return an iterable of (tokentype, value) pairs generated from
@@ -349,7 +376,7 @@ def _pygments_get_tokens_preprocess(self, text, unfiltered=False):
     Also preprocess the text, i.e. expand tabs and strip it if
     wanted and applies registered filters.
     """
-    if not isinstance(text, text_type):
+    if not isinstance(text, str):
         if self.encoding == "guess":
             text, _ = guess_decode(text)
         elif self.encoding == "chardet":
@@ -380,7 +407,7 @@ def _pygments_get_tokens_preprocess(self, text, unfiltered=False):
         if text.startswith(u"\ufeff"):
             text = text[len(u"\ufeff") :]
 
-    # text now *is* a Unicode string
+    # text now *is* a unicode string
     text = text.replace("\r\n", "\n")
     text = text.replace("\r", "\n")
     if self.stripall:
@@ -391,12 +418,13 @@ def _pygments_get_tokens_preprocess(self, text, unfiltered=False):
         text = text.expandtabs(self.tabsize)
     if self.ensurenl and not text.endswith("\n"):
         text += "\n"
+
     # EDIT: This is not from the original Pygments code. It was added to return
     # the preprocessed text.
     return text
 
 
-# This code was copied from ``pygments.lexer.Lexer.get_token``, v. 2.1.3 (the last
+# This code was copied from ``pygments.lexer.Lexer.get_token``, v. 2.6.1 (the last
 # few lines).
 def _pygments_get_tokens_postprocess(self, text, unfiltered=False):
     def streamer():
@@ -936,7 +964,7 @@ def _remove_comment_delim(
         # Special case: COBOL. A comment has the character ``*`` or ``/`` in column 7.
         if lexer.name == "COBOL":
             return string[7:] if string[6] in ("*", "/") else string
-        # Unline the opening and closing block comment delimiters, the inline comment delimiter may be a sequence. Check each possilibty for a match.
+        # Unlike the opening and closing block comment delimiters, the inline comment delimiter may be a sequence. Check each possibility for a match.
         inline_comment_delim_seq = comment_delim_info[0]
         # Look at each possibility for a match.
         string_lower = string.lower()
